/**
 * @file gthr-l4min.h
 * @author Rudolf Weber Ravensburg rw@infnet.verein.de
 * Copyright GNU
 *
 * Minimal gthread environment on L4 
 */
#ifndef GTHR_L4MIN_H
#define GTHR_L4MIN_H

#define __GTHREADS 1
#include<l4/util/lock.h>

typedef l4_simple_lock_t __gthread_mutex_t;
typedef struct
{
  l4_simple_lock_t mutex;
  int once;
} __gthread_once_t;

#define __GTHREAD_MUTEX_INIT 0
#define __GTHREAD_ONCE_INIT {0,0}
/**
 * our thread system is active
 */
static inline int __gthread_active_p()
{
  return 1;
}

static inline int __gthread_mutex_lock (__gthread_mutex_t *mutex)
{
  l4_simple_lock(mutex);
  return 0;
}

static inline int __gthread_mutex_trylock (__gthread_mutex_t *mutex)
{
  return l4_simple_try_lock(mutex);
}

static inline int __gthread_mutex_unlock (__gthread_mutex_t *mutex)
{
  l4_simple_lock_locked(mutex);
  return 0;
}

static inline int __gthread_once (__gthread_once_t *once, void (*func) ())
{
  if(once->once == 0)
  {
    __gthread_mutex_lock(&(once->mutex));
    if(once->once==0)
    {
      (*func)();
      once->once++;
    }
    __gthread_mutex_unlock(&(once->mutex));
  }
  return 0;
}
#endif

